#!/usr/bin/perl
use warnings;
use strict;
use lib do { __FILE__ =~ m|^(.*?)[^/]*$|; "$1"; };

# Finds the best map from amongst multiple maps generated from
# different random models. Uses the sum over the Fourier Shell
# Coefficient (FSC) as measure of quality.

# Input:
#   $listing_file - file containing names of auto3dem log files

my ($listing_file) = @ARGV;

my @outfiles = ();
open(LIST, $listing_file);
while(<LIST>) {
    chomp;
    if(/\w/) {
	push(@outfiles,$_);
    }
}

my $best_fsc_sum =  0;
my $best_file;

foreach my $outfile (@outfiles) {
    my $fsc_sum = find_fsc_sum($outfile);
    print "$outfile: $fsc_sum\n";
    if($fsc_sum > $best_fsc_sum) {
	$best_fsc_sum = $fsc_sum;
	$best_file = $outfile;
    }
}

print "Best sum(FSC) = $best_fsc_sum (file = $best_file)\n";

# ------------------------------------------------------

sub find_fsc_sum {

    # Extract FSC sum from the final iteration of auto3dem
    #
    # Input:
    #   $file - auto3dem log file
    #
    # Output:
    #   $fsc_sum - sum over FSC

    my ($file) = @_;
    open(FILE, $file);

    my $fsc_sum;

    while(<FILE>) {
	if(/Sum\s+over\s+FSC/) {
	    $fsc_sum = $_;
	}
    }
    $fsc_sum =~ s/\s+$//; # Trim off trailing whitespace
    $fsc_sum =~ s/.*\s//; # Get rid of everthing except resolution
    
    close(FILE);
    return $fsc_sum;
}
