      SUBROUTINE CCF_FFT (FFT1, FFT2, CC_MAP, IDIM1, IDIM2, RES_MIN,
     $     RES_MAX)
      
C     Calculates CC_MAP (array of correlation coefficients between rows
C     and columns of pair of FFTs?)
C
C     AUGUST 1988  PURDUE UNIVERSITY VAX 8550, T. BAKER
C     MODIFIED 17-DEC-1997 BY T. BAKER (Rid of EM.CMM)
C     MODIFIED  2-MAR-1998 BY T. BAKER (Make UNIX compatible)
C     MODIFIED  2-MAR-1998 BY RSS - merged with ccf
C     MODIFIED 22-AUG-2006 BT R SINKOVITS, CLEANUP AND MODERNIZATION

      IMPLICIT NONE
      
      INTEGER*4, INTENT(IN) :: IDIM1, IDIM2
      REAL*4, INTENT(IN)    :: FFT1(IDIM1*2,IDIM2), FFT2(IDIM1*2,IDIM2)
      REAL*4, INTENT(IN)    :: RES_MIN, RES_MAX
      REAL*4, INTENT(OUT)   :: CC_MAP(IDIM1,IDIM2)

      INTEGER*4 I, II, J, KK, K, L
      INTEGER*4 IY, IYMAX, II1, II2, JJ, ALLOC_STAT
      REAL*4    WORK, RSQ, YSQ, DIMDIM, DIV
      REAL*4    RES_MINSQ, RES_MAXSQ
      LOGICAL   RESTEST
      real*4, allocatable :: COL_ARRAY(:), ROW_ARRAY(:), TMP(:,:)
      
      allocate(tmp(2*idim1,idim2), col_array(2*idim2), 
     1     row_array(2*idim1), stat = alloc_stat)
      
      if(alloc_stat > 0) then
         write(*,*) 'Memory allocation failed in CCF_FFT'
         stop
      endif
      
      CC_MAP = 0.0 !Fortran 90 array syntax
      
      IYMAX = (IDIM2/2)+1
      IF(RES_MIN <= 0.0 .AND. RES_MAX <= 0.0) THEN
         RESTEST = .FALSE.
      ELSE
         RESTEST = .TRUE.
         IF(RES_MAX < FLOAT(IYMAX)) IYMAX = NINT(RES_MAX)
         RES_MINSQ = RES_MIN*RES_MIN
         RES_MAXSQ = RES_MAX*RES_MAX
      ENDIF

C     TRANSFORM FIRST DIMENSION  ((IDIM2/2)+1 ROWS)

      IY     = 0
      DIMDIM = FLOAT(IDIM1)*FLOAT(IDIM2)
      DIV    = FLOAT(IDIM2/2) + 1.0

      DO J = 1,((IDIM2/2)+1)
         IY = IY+1
         IF(IY <= IYMAX .AND. FLOAT(IY) <= RES_MAX) THEN
            DO II = 1,(2*IDIM1),2
               ROW_ARRAY(II)   = (FFT1(II,J)*FFT2(II,J) +
     1              FFT1(II+1,J)*FFT2(II+1,J))/DIMDIM
               ROW_ARRAY(II+1) = (FFT1(II,J)*FFT2(II+1,J) -
     1              FFT1(II+1,J)*FFT2(II,J))/DIMDIM
            ENDDO
            IF(RESTEST) THEN
               YSQ = (IY-1)**2.0 ! RESOLUTION TEST
               DO K = 1,((IDIM1/2)+1)
                  RSQ = YSQ+(K-1)**2.0
                  IF(RSQ > RES_MAXSQ .OR. RSQ < RES_MINSQ) THEN
                     KK              = 2*K-1
                     ROW_ARRAY(KK)   = 0.0
                     ROW_ARRAY(KK+1) = 0.0
                     IF(K.NE.1) THEN
                        ROW_ARRAY((2*IDIM1+3)-KK) = 0.0
                        ROW_ARRAY((2*IDIM1+2)-KK) = 0.0
                     ENDIF
                  ENDIF
               ENDDO
            ENDIF

C     TRANSFORM THE ROW AND STORE IT IN TMP
            CALL FOURT (ROW_ARRAY, IDIM1, 1, -1, 1, WORK)
            DO L = 1,IDIM1*2
               TMP(L,J) = ROW_ARRAY(L)
            ENDDO
         ELSE
            DO L = 1,IDIM1*2    ! ZERO REST OF ARRAY
               TMP(L,J) = 0.0
            ENDDO
         ENDIF
         
      ENDDO
      
      DO I = 1,IDIM1            ! MAIN LOOP OF FFT
         II1 = 2*I - 1
         II2 = II1 + 1
         JJ  = 1
         DO J = 1,(IDIM2/2)+1
            COL_ARRAY(JJ)   = TMP(II1,J)
            COL_ARRAY(JJ+1) = TMP(II2,J)
            JJ              = JJ + 2
         ENDDO

C     SET FRIEDEL MATES
         K = IDIM2 - 1
         DO J = (IDIM2+3),(2*IDIM2),2
            COL_ARRAY(J)   =  COL_ARRAY(K)
            COL_ARRAY(J+1) = -COL_ARRAY(K+1)
            K              = K-2
         ENDDO
         CALL FOURT (COL_ARRAY, IDIM2, 1, -1, 1, WORK) ! TRANSFORM COLUMN
         DO J = 1,IDIM2
            CC_MAP(I,J) = COL_ARRAY(2*J-1)
         ENDDO
         
      ENDDO                     ! MAIN LOOP OF FFT
      
      deallocate(tmp, col_array, row_array)
      
      RETURN
      END SUBROUTINE CCF_FFT
